
local myname, mm = ...
local _G = getfenv(0)

local err = mm.err

local pairs = _G.pairs
local random = _G.random
local select = _G.select
local strtrim = _G.strtrim
local tonumber = _G.tonumber


-- for Debug
_G.MountMacro = mm

--[==[
	--------------------
	Addon initiatisation
	--------------------
  ]==]

local f = _G.CreateFrame("frame")
f:SetScript("OnEvent", function(self, event, ...) if mm[event] then return mm[event](mm, event, ...) end end)


function mm:RegisterEvent(...) for i=1,select("#", ...) do f:RegisterEvent((select(i, ...))) end end
function mm:UnregisterEvent(...) for i=1,select("#", ...) do f:UnregisterEvent((select(i, ...))) end end
mm.RegisterEvents, mm.UnregisterEvents = mm.RegisterEvent, mm.UnregisterEvent

mm:RegisterEvent("ADDON_LOADED")
function mm:ADDON_LOADED(event, addon)
	if addon ~= myname then return end
	self:InitDB()

	-- Do anything you need to do after addon has loaded

	_G.LibStub("tekKonfig-AboutPanel").new(nil, myname) -- Make first arg nil if no parent config panel

	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil

	if _G.IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
end


function mm:PLAYER_LOGIN()

	self:setDefaultChanelForPrint()

	-- Register the events needed
	self:RegisterEvent("PLAYER_LOGOUT")
	self:RegisterEvent("UNIT_SPELLCAST_START")
	--self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("ZONE_CHANGED")


	-- Do anything you need to do after the player has entered the world
	--self.db.current_mount = 
	self:RefreshMountList()
	self:RefreshMacro()


	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end


function mm:PLAYER_LOGOUT()
	self:FlushDB()
	-- Do anything you need to do as the player logs out
end

--[==[
	--------------
	Main functions
	--------------
  ]==]

--[[  ==========================================
		GetRandomMount

		Select a random mount for particular mount type. 
	
		Valid mount types are:

		g 			= Ground
		f 			= Flying
		fg			= Ground + Flying that can also be used on the ground
		ww 		= Water Walking
		uw 		= Under Water
		tff 		= Transport Friend while Flying
		tfg		= Transport Friend on the ground
		r 			= Repair
		vashjir	= Can only be used in Vashj'Ir while swimming
		ahnqiraj	= Can only be used in the Temple of Ahn'Qiraj
]]		

function mm:GetRandomMount(mountType, zoneID)
	-- Special case for the current mount
	if mountType == 'cm' then return self.db.current_mount or "None" end

	local mounts_id, mounts_name, mounts_isfav = self.db.ml_id, self.db.ml_name, self.db.ml_isfav

	-- Speciel cases based on zones
	if mountType == 'g' and zoneID == 766 then
		-- Temple of Ahn'Qiraj, we need to choose one of the beatles
		mountType = 'ahnqiraj'
	elseif mountType == 'uw' and (zoneID == 610 or zoneID == 613 or zoneID == 614 or zoneID == 615) then
	-- Vashj'Ir
		if #mounts_id.vashjir > 0 then
			-- First, choose the Seahorse
			mountType = 'vashjir'
		end
	end


	for _=1,1000 do
		local i = random(1,#mounts_id[mountType])
		if mounts_isfav[mountType][i] or random() < mm.dbpc.weight_for_nonfevarite then
			return mounts_name[mountType][i], mounts_id[mountType][i], mounts_isfav[mountType][i]
		end
	end

	-- No match were found
	return 'None Found'
end

--[[	==========================================
		RefreshMacro

		Template and functions to refreach the mount macros.

		For now, everything is hardcoded and there is only one macro.

		The refresh will always happen out of combat.
]]


do -- closure

local macro_template = {
	default = [=[
#showtooltip
/cast [mod:shift]<ww>;[mod:ctrl][flyable,nomounted]<f>;[swimming,nomounted]<uw>;[nomounted]<g>
/dismount [mounted]
/stopmacro
/cast <cm>
]=],
	SHAMAN = [=[
#showtooltip
/cast [mod:shift]<ww>;[mod:ctrl][flyable,nomounted]<f>;[swimming,nomounted]<uw>;[nomounted]<g>
/dismount [mounted]
/stopmacro
/cast <cm>
]=],
}

local macro_tags = {
	['<g>']		= 'g',
	['<f>']		= 'f',
	['<fg>']		= 'fg',
	['<r>']		= 'r',
	['<uw>']		= 'uw',
	['<ww>']		= 'ww',
	['<pf>']		= 'tff',
	['<pg>']		= 'tfg',
	['<pfg>']	= 'tfg',
	['<cm>']		= 'cm',
}


local THROTTLE_TIME = 0.4
local throt
local f = _G.CreateFrame("Frame")
f:Hide()
f:SetScript("OnShow", function(self)
	throt = THROTTLE_TIME
end)
f:SetScript("OnUpdate", function(self, elapsed)
	throt = throt - elapsed
	if throt < 0 then
		if _G.InCombatLockdown() then
			throt = throt + THROTTLE_TIME
		else
			mm:RefreshMacro()
			f:Hide()
		end
	end
end)

local function RefreshMacro_bucket()
	f:Show()
end

function mm:RefreshMacro()
	if _G.InCombatLockdown() then
		RefreshMacro_bucket()
		return
	end

--	mm.dbpc.mm1 = mm.dbpc.mm1 or macro_template[select(2,_G.UnitClass("player"))] or macro_template.default
	mm.dbpc.mm1 = macro_template[select(2,_G.UnitClass("player"))] or macro_template.default
	local macro_text = mm.dbpc.mm1
	local zoneID = _G.GetCurrentMapAreaID()

	-- Replace each tags with a random mount
	for tag, mountType in pairs(macro_tags) do
		macro_text = macro_text:gsub(tag, self:GetRandomMount(mountType, zoneID))
	end

	-- Verify if the macro already exist otherwise create it
	local macroIndex = _G.GetMacroIndexByName("mm")
	if macroIndex == 0 then
		macroIndex = _G.CreateMacro("mm", "INV_MISC_QUESTIONMARK", "", true)
	end

	-- Update the macro
	macroIndex = _G.EditMacro(macroIndex, "mm", "INV_MISC_QUESTIONMARK", strtrim(macro_text))
	if not macroIndex then
		err("Could not update the macro [[%S]", strtrim(macro_text))
	end

end

end -- closure

--[==[
	------
	Events
	------
  ]==]

function mm:UNIT_SPELLCAST_START(event, unit, spellName, rank, spellLineID, spellID)
	if self.db.is_a_mount[tonumber(spellID)] then
		self.db.current_mount = _G.GetSpellInfo(spellID)
		self:RefreshMacro()
	end
end

-- function mm:UNIT_SPELLCAST_SUCCEEDED(event, unit, spellName, rank, spellLineID, spellID)
-- 	if self.db.is_a_mount[tonumber(spellID)] then
-- 		self.db.current_mount = GetSpellInfo(spellID)
-- 	end
-- end

local current_zone
function mm:ZONE_CHANGED()
	local zoneID = _G.GetCurrentMapAreaID()
	if current_zone ~= zoneID then
		current_zone = zoneID
		self:RefreshMacro()
	end
end
